@@ -352,6 +352,15 @@ func convertServerConfig(agentConfig *Config) (*nomad.Config, error) {
 	conf.JobMaxPriority = jobMaxPriority
 	conf.JobDefaultPriority = jobDefaultPriority
 
+	jobMaxCount := structs.JobDefaultMaxCount
+	if agentConfig.Server.JobMaxCount != nil {
+		jobMaxCount = *agentConfig.Server.JobMaxCount
+		if jobMaxCount < 0 {
+			return nil, fmt.Errorf("job_max_count (%d) cannot be negative", jobMaxCount)
+		}
+	}
+	conf.JobMaxCount = jobMaxCount
+
 	if agentConfig.Server.JobTrackedVersions != nil {
 		if *agentConfig.Server.JobTrackedVersions <= 0 {
 			return nil, fmt.Errorf("job_tracked_versions must be greater than 0")
@@ -655,8 +664,28 @@ func convertServerConfig(agentConfig *Config) (*nomad.Config, error) {
 		return nil, fmt.Errorf("number of schedulers should be between 0 and %d",
 			runtime.NumCPU())
 	}
+
+	// If the operator has specified a client introduction server config block,
+	// translate this into the internal server configuration object.
+	if agentConfig.Server.ClientIntroduction != nil {
+		if agentConfig.Server.ClientIntroduction.Enforcement != "" {
+			conf.NodeIntroductionConfig.Enforcement = agentConfig.Server.ClientIntroduction.Enforcement
+		}
+		if agentConfig.Server.ClientIntroduction.DefaultIdentityTTL > 0 {
+			conf.NodeIntroductionConfig.DefaultIdentityTTL = agentConfig.Server.ClientIntroduction.DefaultIdentityTTL
+		}
+		if agentConfig.Server.ClientIntroduction.MaxIdentityTTL > 0 {
+			conf.NodeIntroductionConfig.MaxIdentityTTL = agentConfig.Server.ClientIntroduction.MaxIdentityTTL
+		}
+	}
+
+	if err := conf.NodeIntroductionConfig.Validate(); err != nil {
+		return nil, fmt.Errorf("invalid server.client_introduction configuration: %w", err)
+	}
+
 	// Copy LogFile config value
 	conf.LogFile = agentConfig.LogFile
+
 	return conf, nil
 }
 
@@ -742,6 +771,47 @@ func (a *Agent) finalizeClientConfig(c *clientconfig.Config) error {
 		to configure Nomad to work with Consul.`)
 	}
 
+	// If the operator has not set an intro token via the CLI or an environment
+	// variable, attempt to read the intro token from the file system. This
+	// cannot be used as a CLI override.
+	if c.IntroToken == "" {
+		if err := a.readIntroTokenFile(c); err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
+// readIntroTokenFile attempts to read the intro token from the file system.
+func (a *Agent) readIntroTokenFile(cfg *clientconfig.Config) error {
+
+	rootFile, err := os.OpenInRoot(cfg.StateDir, "intro_token.jwt")
+	if err != nil {
+		if os.IsNotExist(err) {
+			return nil
+		}
+		return err
+	}
+
+	fileStat, err := rootFile.Stat()
+	if err != nil {
+		return fmt.Errorf("failed to stat intro token file: %w", err)
+	}
+
+	// If the file exists and is a file, attempt to read the contents and set
+	// the intro token. Any error is logged for the operator to investigate but
+	// does not block the agent from starting.
+	if fileStat.IsDir() {
+		return fmt.Errorf("intro token file is a directory")
+	}
+
+	content, err := helper.ReadFileContent(rootFile)
+	if err != nil {
+		return fmt.Errorf("failed to read intro token file: %w", err)
+	}
+
+	cfg.IntroToken = strings.TrimSpace(string(content))
 	return nil
 }
 
@@ -757,6 +827,7 @@ func convertClientConfig(agentConfig *Config) (*clientconfig.Config, error) {
 	conf.Servers = agentConfig.Client.Servers
 	conf.DevMode = agentConfig.DevMode
 	conf.EnableDebug = agentConfig.EnableDebug
+	conf.IntroToken = agentConfig.Client.IntroToken
 
 	if agentConfig.Region != "" {
 		conf.Region = agentConfig.Region
@@ -766,6 +837,7 @@ func convertClientConfig(agentConfig *Config) (*clientconfig.Config, error) {
 		conf.AllocDir = filepath.Join(agentConfig.DataDir, "alloc")
 		conf.HostVolumesDir = filepath.Join(agentConfig.DataDir, "host_volumes")
 		conf.HostVolumePluginDir = filepath.Join(agentConfig.DataDir, "host_volume_plugins")
+		conf.CommonPluginDir = filepath.Join(agentConfig.DataDir, "common_plugins")
 		dataParent := filepath.Dir(agentConfig.DataDir)
 		conf.AllocMountsDir = filepath.Join(dataParent, "alloc_mounts")
 	}
@@ -784,6 +856,9 @@ func convertClientConfig(agentConfig *Config) (*clientconfig.Config, error) {
 	if agentConfig.Client.HostVolumesDir != "" {
 		conf.HostVolumesDir = agentConfig.Client.HostVolumesDir
 	}
+	if agentConfig.Client.CommonPluginDir != "" {
+		conf.CommonPluginDir = agentConfig.Client.CommonPluginDir
+	}
 	if agentConfig.Client.NetworkInterface != "" {
 		conf.NetworkInterface = agentConfig.Client.NetworkInterface
 	}
@@ -868,17 +943,6 @@ func convertClientConfig(agentConfig *Config) (*clientconfig.Config, error) {
 	// Canonicalize Node struct
 	conf.Node.Canonicalize()
 
-	// Reserve resources on the node.
-	// COMPAT(0.10): Remove in 0.10
-	r := conf.Node.Reserved
-	if r == nil {
-		r = new(structs.Resources)
-		conf.Node.Reserved = r
-	}
-	r.CPU = agentConfig.Client.Reserved.CPU
-	r.MemoryMB = agentConfig.Client.Reserved.MemoryMB
-	r.DiskMB = agentConfig.Client.Reserved.DiskMB
-
 	res := conf.Node.ReservedResources
 	if res == nil {
 		res = new(structs.NodeReservedResources)
