@@ -119,6 +119,7 @@ func (c *Command) readConfig() *Config {
 	flags.StringVar(&cmdConfig.Client.NetworkInterface, "network-interface", "", "")
 	flags.StringVar((*string)(&cmdConfig.Client.PreferredAddressFamily), "preferred-address-family", "", "ipv4 or ipv6")
 	flags.IntVar(&cmdConfig.Client.NetworkSpeed, "network-speed", 0, "")
+	flags.StringVar(&cmdConfig.Client.IntroToken, "client-intro-token", "", "")
 
 	// General options
 	flags.Var((*flaghelper.StringFlag)(&configPath), "config", "config")
@@ -225,6 +226,12 @@ func (c *Command) readConfig() *Config {
 		}
 	}
 
+	// Perform an environment look for the client bootstrap token. If this is
+	// present, it will override the CLI flag.
+	if envToken, found := os.LookupEnv("NOMAD_CLIENT_INTRO_TOKEN"); found {
+		cmdConfig.Client.IntroToken = envToken
+	}
+
 	// Load the configuration
 	var config *Config
 
@@ -1074,7 +1081,7 @@ func (c *Command) handleSignals() int {
 	signalCh := make(chan os.Signal, 4)
 	defer signal.Stop(signalCh)
 
-	signal.Notify(signalCh, os.Interrupt, syscall.SIGTERM, syscall.SIGHUP, syscall.SIGPIPE)
+	signal.Notify(signalCh, syscall.SIGINT, syscall.SIGTERM, syscall.SIGHUP, syscall.SIGPIPE)
 
 	// Signal readiness only once signal handlers are setup
 	sdSock, err := openNotify()
@@ -1113,7 +1120,7 @@ func (c *Command) handleSignals() int {
 				}
 
 				return c.terminateGracefully(signalCh, sdSock)
-			case os.Interrupt:
+			case syscall.SIGINT:
 				if !c.agent.GetConfig().LeaveOnInt {
 					return 1
 				}
@@ -1506,8 +1513,8 @@ General Options (clients and servers):
    Enable sending Nomad agent logs to the Windows Event Log.
 
   -eventlog-level
-	Specifies the verbosity of logs the Nomad agent outputs. Valid log levels
-	include ERROR, WARN, or INFO in  order of verbosity. Level must be
+    Specifies the verbosity of logs the Nomad agent outputs. Valid log levels
+    include ERROR, WARN, or INFO in  order of verbosity. Level must be
     of equal or less verbosity as defined for the -log-level parameter.
 
   -node=<name>
@@ -1634,6 +1641,13 @@ Client Options:
     Directory containing dynamic host volume plugins. The default is
     <data-dir>/host_volume_plugins.
 
+  -client-intro-token
+    The JWT token used to authenticate with servers during the client's initial
+    registration. You may also set the token via the "NOMAD_CLIENT_INTRO_TOKEN"
+    environment variable, which overrides this flag. If neither are set, the
+    agent looks for an "intro_token.jwt" file within the client state
+    directory.
+
 ACL Options:
 
   -acl-enabled
