@@ -4,11 +4,11 @@
 package util
 
 func CalculateCPUPercent(newSample, oldSample, newTotal, oldTotal uint64, cores int) float64 {
-	numerator := newSample - oldSample
-	denom := newTotal - oldTotal
-	if numerator <= 0 || denom <= 0 {
+	if (newSample <= oldSample) || (newTotal <= oldTotal) {
 		return 0.0
 	}
+	numerator := newSample - oldSample
+	denom := newTotal - oldTotal
 
 	return (float64(numerator) / float64(denom)) * float64(cores) * 100.0
 }
