@@ -44,12 +44,14 @@ import (
 	"github.com/hashicorp/nomad/nomad/deploymentwatcher"
 	"github.com/hashicorp/nomad/nomad/drainer"
 	"github.com/hashicorp/nomad/nomad/lock"
+	"github.com/hashicorp/nomad/nomad/peers"
 	"github.com/hashicorp/nomad/nomad/reporting"
 	"github.com/hashicorp/nomad/nomad/state"
 	"github.com/hashicorp/nomad/nomad/structs"
 	"github.com/hashicorp/nomad/nomad/structs/config"
 	"github.com/hashicorp/nomad/nomad/volumewatcher"
 	"github.com/hashicorp/nomad/scheduler"
+	sstructs "github.com/hashicorp/nomad/scheduler/structs"
 )
 
 const (
@@ -172,8 +174,8 @@ type Server struct {
 
 	// peers is used to track the known Nomad servers. This is
 	// used for region forwarding and clustering.
-	peers      map[string][]*serverParts
-	localPeers map[raft.ServerAddress]*serverParts
+	peers      map[string][]*peers.Parts
+	localPeers map[raft.ServerAddress]*peers.Parts
 	peerLock   sync.RWMutex
 
 	// serf is the Serf cluster containing only Nomad
@@ -181,6 +183,10 @@ type Server struct {
 	// and automatic clustering within regions.
 	serf *serf.Serf
 
+	// peersCache is used to cache the parsed Nomad server member peer parts.
+	// This is used to avoid re-parsing the Serf tags on every access.
+	peersCache *peers.PeerCache
+
 	// bootstrapped indicates if Server has bootstrapped or not.
 	bootstrapped *atomic.Bool
 
@@ -351,8 +357,9 @@ func NewServer(config *Config, consulCatalog consul.CatalogAPI, consulConfigFunc
 		rpcServer:               rpc.NewServer(),
 		streamingRpcs:           structs.NewStreamingRpcRegistry(),
 		nodeConns:               make(map[string][]*nodeConnState),
-		peers:                   make(map[string][]*serverParts),
-		localPeers:              make(map[raft.ServerAddress]*serverParts),
+		peers:                   make(map[string][]*peers.Parts),
+		localPeers:              make(map[raft.ServerAddress]*peers.Parts),
+		peersCache:              peers.NewPeerCache(),
 		bootstrapped:            &atomic.Bool{},
 		reassertLeaderCh:        make(chan chan error),
 		reconcileCh:             make(chan serf.Member, 32),
@@ -1281,6 +1288,7 @@ func (s *Server) setupRpcServer(server *rpc.Server, ctx *RPCContext) {
 	// These endpoints are client RPCs and don't include a connection context
 	_ = server.Register(NewClientStatsEndpoint(s))
 	_ = server.Register(newNodeMetaEndpoint(s))
+	_ = server.Register(newNodeIdentityEndpoint(s))
 
 	// These endpoints have their streaming component registered in
 	// setupStreamingEndpoints, but their non-streaming RPCs are registered
@@ -1863,7 +1871,7 @@ func (s *Server) listenWorkerEvents() {
 			}
 		case e := <-s.workersEventCh:
 			switch event := e.(type) {
-			case *scheduler.PortCollisionEvent:
+			case *sstructs.PortCollisionEvent:
 				if event == nil || event.Node == nil {
 					continue
 				}
@@ -2188,7 +2196,7 @@ directory.
 The "address" field is the address and port of the server.
 The "non_voter" field controls whether the server is a non-voter, which is used
 in some advanced Autopilot configurations, please see
-https://developer.hashicorp.com/nomad/tutorials/manage-clusters/outage-recovery for more information. If
+https://developer.hashicorp.com/nomad/docs/manage/outage-recovery for more information. If
 "non_voter" is omitted it will default to false, which is typical for most
 clusters.
 
@@ -2205,5 +2213,5 @@ creating the peers.json file, and that all servers receive the same
 configuration. Once the peers.json file is successfully ingested and applied, it
 will be deleted.
 
-Please see https://developer.hashicorp.com/nomad/tutorials/manage-clusters/outage-recovery for more information.
+Please see https://developer.hashicorp.com/nomad/docs/manage/outage-recovery for more information.
 `
